#!/usr/bin/perl -w

# log.pl from os.inf.tu-dresden.de CVS repo
#
#  adam@os.inf.tu-dresden.de

use strict;

my $cvsroot = $ENV{'CVSROOT'};

my $users = '';
my $logfile;
my $donefiles;
my @files;

my $VIEWCVSURL = 'http://cvs.alioth.debian.org/cgi-bin/cvsweb.cgi';
my $URLSUFFIX  = 'cvsroot=minicom';

# turn off setgid
#
$) = $(;

# parse command line arguments
#
while (@ARGV) {
        my $arg = shift @ARGV;

	if ($arg eq '-m') {
                $users = "$users " . shift @ARGV;
	} elsif ($arg eq '-f') {
		($logfile) && die "Too many '-f' args";
		$logfile = shift @ARGV;
	} else {
		($donefiles) && die "Too many arguments!\n";
		$donefiles = 1;
		@files = split(/ /, $arg);
	}
}

my $srepos = shift @files;

# Some date and time arrays
#
my @mos = qw/January February March April May June July August September 
           October November December/;
my @days = qw/Sunday Monday Tuesday Wednesday Thursday Friday Saturday/;

my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime;

# get login name
#
my $login = getlogin || (getpwuid($<))[0] || "nobody";

my $MAIL = 0;
# open log file for appending
#
if ($logfile) {
  open(OUT, ">>" . $logfile) || die "Could not open(" . $logfile . "): $!\n";
}
my $longyear = 1900 + $year;

my $maillogmsg = '';

my $date_author = "Date:\t$days[$wday] $mos[$mon] $mday, $longyear @ $hour:" . sprintf("%02d", $min) . "\n".
		  "Author:\t$login\n\n";

# print out the log Header
# 
if ($logfile) {
  print OUT "\n";
  print OUT "**************************************\n";
  print OUT $date_author;
}

# print the stuff from logmsg that comes in on stdin to the logfile
#
open(IN, "-") || die "Couldn't open STDIN!\n";
my $branch_path     = '';
my $branch_subj_tag = '';
while (<IN>) {
	print OUT $_ if $logfile;
	$maillogmsg .= $_;
	if (/^\s+Tag: (\S+)/) {
		$branch_subj_tag = " [$1]";
		$branch_path = 'Attic/';
	}
}
close(IN);

my $mailcmd = "| mailx -s 'CVS update: $srepos$branch_subj_tag'";

# open mail command
if ($users) {
	$mailcmd = "$mailcmd $users";
	$MAIL = 1;
	open(MAIL, $mailcmd) || die "Could not Exec($mailcmd): $!\n";
	print MAIL "\n".$date_author.$maillogmsg;
}

print OUT "\n" if defined $logfile;

my $firstline = 1;
while (@files) {
	my $file = shift @files;
	if ($file eq "-") {
		print OUT  "[input file was '-']\n" if $logfile;
		print MAIL "[input file was '-']\n" if $MAIL;
		last;
	}

	if ($MAIL && $firstline) {
	  print MAIL "\n";
	  $firstline = 0;
	}

	if ($file !~ /^([^,]+),([^,]+),([^,]+)$/) {
	  print OUT  "$0: Error in argument ($file)!\n" if $logfile;
	  print MAIL "$0: Error in argument ($file)!\n" if $MAIL;
	  next;
	}
	my ($filename, $oldver, $newver) = ($1, $2, $3);
	my $filepath = "$srepos/$branch_path$filename";
	if ($newver eq 'NONE') {
	  print MAIL "$filename:\tRemoved.\n" if $MAIL;
	  print OUT  "$filename:\tRemoved.\n" if $logfile;
	} elsif ($oldver eq 'NONE') {
	  print MAIL "$filename:\t$VIEWCVSURL/$filepath?rev=$newver&content-type=text/x-cvsweb-markup&$URLSUFFIX\n" if $MAIL;
	  print OUT "$filename:\tAdded version $newver.\n" if $logfile;
	} else {
	  print MAIL "$filename:\t$VIEWCVSURL/$filepath.diff?r1=$oldver&r2=$newver&$URLSUFFIX\n" if $MAIL;
	  print OUT "$filename:\t$oldver => $newver\n" if $logfile;
	}
}

if ($logfile) {
  close(OUT)  || die "Write to $logfile failed";
}
if ($logfile) {
  close(MAIL) || die "Pipe to $mailcmd failed";
}

exit 0;
